@@ -35,7 +35,7 @@ your system.
 
 use XSLoader;
 
-our $VERSION = '0.07_90';
+our $VERSION = '0.12_90';
 
 XSLoader::load( __PACKAGE__, $VERSION );
 
@@ -52,41 +52,62 @@ Instantiates I<CLASS>.
 Sets 1 or more globals in I<OBJ>. See below for details on type conversions
 from Perl to JavaScript.
 
+Returns I<OBJ>.
+
 =head2 $obj = I<OBJ>->helpers()
 
 Defines QuickJS’s “helpers”, e.g., C<console.log>.
 
+Returns I<OBJ>.
+
 =head2 $obj = I<OBJ>->std()
 
 Enables (but does I<not> import) QuickJS’s C<std> module.
 
+Returns I<OBJ>.
+
 =head2 $obj = I<OBJ>->os()
 
 Like C<std()> but for QuickJS’s C<os> module.
 
 =head2 $VALUE = I<OBJ>->eval( $JS_CODE )
 
-Comparable to running C<qjs -e '...'>. Returns the last value from $JS_CODE;
+Comparable to running C<qjs -e '...'>. Returns $JS_CODE’s last value;
 see below for details on type conversions from JavaScript to Perl.
 
 Untrapped exceptions in JavaScript will be rethrown as Perl exceptions.
 
 $JS_CODE is a I<character> string.
 
-=head2 I<OBJ>->eval_module( $JS_CODE )
+=head2 $obj = I<OBJ>->eval_module( $JS_CODE )
 
 Runs $JS_CODE as a module, which enables ES6 module syntax.
 Note that no values can be returned directly in this mode of execution.
 
+Returns I<OBJ>.
+
+=head2 $obj = I<OBJ>->await()
+
+Blocks until all of I<OBJ>’s pending work (if any) is complete.
+
+For example, if you C<eval()> some code that creates a promise, call
+this to wait for that promise to complete.
+
+Returns I<OBJ>.
+
 =head2 $obj = I<OBJ>->set_module_base( $PATH )
 
 Sets a base path (a byte string) for ES6 module imports.
 
+Returns I<OBJ>.
+
 =head2 $obj = I<OBJ>->unset_module_base()
 
 Restores QuickJS’s default directory for ES6 module imports
 (as of this writing, it’s the process’s current directory).
 
+Returns I<OBJ>.
+
 =cut
 
 # ----------------------------------------------------------------------
@@ -111,7 +132,9 @@ This module converts returned values from JavaScript thus:
 
 =item * “Plain” objects become Perl hash references.
 
-=item * Functions become Perl code references.
+=item * Function objects become Perl L<JavaScript::QuickJS::Function> objects.
+
+=item * RegExp objects become Perl L<JavaScript::QuickJS::RegExp> objects.
 
 =item * Behaviour is B<UNDEFINED> for other object types.
 
@@ -121,16 +144,18 @@ This module converts returned values from JavaScript thus:
 
 =head1 TYPE CONVERSION: PERL → JAVASCRIPT
 
-Generally speaking, it’s the inverse of JS → Perl, though since Perl doesn’t
-differentiate “numeric strings” from “numbers” there’s occasional ambiguity.
-In such cases, behavior is undefined; be sure to typecast in JavaScript
-accordingly.
+Generally speaking, it’s the inverse of JS → Perl:
 
 =over
 
 =item * Perl strings, numbers, & booleans become corresponding JavaScript
 primitives.
 
+B<IMPORTANT:> Perl versions before 5.36 don’t reliably distinguish “numeric
+strings” from “numbers”. If your perl predates 5.36, typecast accordingly
+to prevent your Perl “number” from becoming a JavaScript string. (Even in
+5.36 and later it’s still a good idea.)
+
 =item * Perl undef becomes JS null.
 
 =item * Unblessed array & hash references become JavaScript arrays and
@@ -140,6 +165,12 @@ primitives.
 
 =item * Perl code references become JavaScript functions.
 
+=item * L<JavaScript::QuickJS::Function> objects become their original
+JavaScript C<Function> objects.
+
+=item * L<JavaScript::QuickJS::RegExp> objects become their original
+JavaScript C<RegExp> objects.
+
 =item * Anything else triggers an exception.
 
 =back
@@ -150,14 +181,9 @@ If any instance of a class of this distribution is DESTROY()ed at Perl’s
 global destruction, we assume that this is a memory leak, and a warning is
 thrown. To prevent this, avoid circular references.
 
-Callbacks make that tricky. As noted above, JavaScript functions
-given to Perl become Perl code references. Those code references are
-closures around the QuickJS context & runtime; once the code reference
-is destroyed, we release its reference to QuickJS.
-
-Perl code references given to JavaScript become JavaScript functions;
-however, QuickJS exposes no facility analogous to Perl C<DESTROY()>. Thus,
-we retain those Perl code references as part of the QuickJS context.
+Callbacks make that tricky. When you give a JavaScript function to Perl,
+that Perl object holds a reference to the QuickJS context. Only once that
+object is C<DESTROY()>ed do we release that QuickJS context reference.
 
 Consider the following:
 
@@ -167,30 +193,40 @@ Consider the following:
 
     $js->eval('__return( a => a )');
 
-Here $js retains a reference to the C<__return> callback. That callback
-refers to C<$return>. Once we run C<eval()>, Perl $return stores
-I<another> callback, which stores a reference to $js. Here we have a
-circular reference. The way to break it is simply:
+This sets $return to be a L<JavaScript::QuickJS::Function> instance. That
+object holds a reference to $js. $js also stores C<__return()>,
+which is a Perl code reference that closes around $return. Thus, we have
+a reference cycle: $return refers to $js, and $js refers to $return. Those
+two values will thus leak, and you’ll see a warning about it at Perl’s
+global destruction time.
+
+To break the reference cycle, just do:
 
     undef $return;
 
-… which is ugly, but it is what it is for now.
+… once you’re done with that variable.
 
-Note also that the C<__return> callback ends with C<()>. Recall that, in
-Perl, a function’s last statement value is the function’s default return
-value. Without the C<()>, then, our callback would return C<$return>,
-which would create yet I<another> reference cycle.
+You I<might> have thought you could instead do:
+
+    $js->set_globals( __return => undef )
+
+… but that doesn’t work because $js holds a reference to all Perl code
+references it B<ever> receives. This is because QuickJS, unlike Perl,
+doesn’t expose object destructors (C<DESTROY()> in Perl), so there’s no
+good way to release that reference to the code reference.
 
 =head1 CHARACTER ENCODING NOTES
 
-Although QuickJS (like all JS engines) assumes its strings are text,
-you can oftentimes pass in byte strings and get a reasonable result.
+QuickJS (like all JS engines) assumes its strings are text. Since Perl
+can’t distinguish text from bytes, though, it’s possible to convert
+Perl byte strings to JavaScript strings. It often yields a reasonable
+result, but not always.
 
 One place where this falls over, though, is ES6 modules. QuickJS, when
 it loads an ES6 module, decodes that module’s string literals to characters.
 Thus, if you pass in byte strings from Perl, QuickJS will treat your
 Perl byte strings’ code points as character code points, and when you
-combine those code points with the ones from your ES6 module you may
+combine those code points with those from your ES6 module you may
 get mangled output.
 
 Another place that may create trouble is if your argument to C<eval()>
@@ -218,11 +254,11 @@ For example, in Perl 5.34 C<print 1000000000000001.0> prints C<1e+15>.
 
 To counteract this loss of precision, add 0 to Perl’s numeric scalars
 (e.g., C<print 0 + 1000000000000001.0>); this will encourage Perl to store
-numbers as integers when possible, which fixes the precision problem.
+numbers as integers when possible, which fixes this precision problem.
 
 =item * Long-double and quad-math perls may lose precision when converting
 numbers to/from JavaScript. To see if this affects your perl—which, if
-you’re unsure, it probably doesn’t—run C<perl -V>, and see if the
+you’re unsure, it probably doesn’t—run C<perl -V>, and see if that perl’s
 compile-time options mention long doubles or quad math.
 
 =back
@@ -234,6 +270,18 @@ QuickJS supports Linux & macOS natively, so these work without issue.
 FreeBSD, OpenBSD, & Cygwin work after a few patches that we apply when
 building this library. (Hopefully these will eventually merge into QuickJS.)
 
+=head1 LIBATOMIC
+
+As of this writing, on RaspiOS this library has to link to libatomic
+in order to avoid runtime linker errors.
+
+To avoid this problem in a portable manner, this library’s F<Makefile.PL>
+tries to detect whether libatomic is necessary and will only link if needed.
+If, for some reason, you need manual control over that linking, set
+C<JS_QUICKJS_LINK_LIBATOMIC> in the environment to 1 or a falsy value.
+
+If you don’t know what any of that means, you can probably ignore it.
+
 =head1 SEE ALSO
 
 Other JavaScript modules on CPAN include:
@@ -258,18 +306,14 @@ L<SpiderMonkey|https://spidermonkey.dev/> engine to Perl.
 
 =head1 LICENSE & COPYRIGHT
 
-Copyright 2022 Gasper Software Consulting.
+This library is copyright 2022 Gasper Software Consulting.
 
 This library is licensed under the same terms as Perl itself.
 See L<perlartistic>.
 
+QuickJS is copyright Fabrice Bellard and Charlie Gordon. It is released
+under the L<MIT license|https://opensource.org/licenses/MIT>.
+
 =cut
 
-#----------------------------------------------------------------------
-
-sub _wrap_jsfunc {
-    my $jsfunc_obj = $_[0];
-    return sub { $jsfunc_obj->call(@_) };
-}
-
 1;
